class TimeIntervalNotSpecifiedException(Exception):
    pass
    
class BadChangeTypeException(Exception):
    pass
    
class NoSuchItemNameException(Exception):
    pass
    
class HangsOnSearchingLoopException(Exception):
    pass